from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .approval_statuses_get_response_results_approval_status import ApprovalStatusesGetResponse_results_approvalStatus
    from .approval_statuses_get_response_results_review import ApprovalStatusesGetResponse_results_review

@dataclass
class ApprovalStatusesGetResponse_results(Parsable):
    # The file’s approval status within a specific review.
    approval_status: Optional[ApprovalStatusesGetResponse_results_approvalStatus] = None
    # Metadata about the review in which this file version was included.
    review: Optional[ApprovalStatusesGetResponse_results_review] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ApprovalStatusesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ApprovalStatusesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ApprovalStatusesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .approval_statuses_get_response_results_approval_status import ApprovalStatusesGetResponse_results_approvalStatus
        from .approval_statuses_get_response_results_review import ApprovalStatusesGetResponse_results_review

        from .approval_statuses_get_response_results_approval_status import ApprovalStatusesGetResponse_results_approvalStatus
        from .approval_statuses_get_response_results_review import ApprovalStatusesGetResponse_results_review

        fields: dict[str, Callable[[Any], None]] = {
            "approvalStatus": lambda n : setattr(self, 'approval_status', n.get_object_value(ApprovalStatusesGetResponse_results_approvalStatus)),
            "review": lambda n : setattr(self, 'review', n.get_object_value(ApprovalStatusesGetResponse_results_review)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("approvalStatus", self.approval_status)
        writer.write_object_value("review", self.review)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class RelationshipsSyncStatusPostResponse_results(Parsable):
    # If set to true, data is available for synchronization using the supplied synchronization token.
    more_data: Optional[bool] = None
    # If set to true, the data returned by the synchronization endpoint can be used to overwrite local copies.
    overwrite: Optional[bool] = None
    # An optional reference passed by the caller and returned by the service.
    reference_id: Optional[str] = None
    # The token that can be used to obtain data via the synchronization endpoint.
    sync_token: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RelationshipsSyncStatusPostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RelationshipsSyncStatusPostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RelationshipsSyncStatusPostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "moreData": lambda n : setattr(self, 'more_data', n.get_bool_value()),
            "overwrite": lambda n : setattr(self, 'overwrite', n.get_bool_value()),
            "referenceId": lambda n : setattr(self, 'reference_id', n.get_str_value()),
            "syncToken": lambda n : setattr(self, 'sync_token', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("moreData", self.more_data)
        writer.write_bool_value("overwrite", self.overwrite)
        writer.write_str_value("referenceId", self.reference_id)
        writer.write_str_value("syncToken", self.sync_token)
    




from enum import Enum

class TransmittalsGetResponse_results_packedStatus(str, Enum):
    SUCCESS = "SUCCESS",
    PARTIAL_SUCCESS = "PARTIAL_SUCCESS",
    FAILED = "FAILED",
    PROCESSING = "PROCESSING",
    EXPIRED = "EXPIRED",
    NOT_ALLOWED = "NOT_ALLOWED",




from enum import Enum

class ViewsGetResponse_modelSetViewVersions_documentVersions_forgeType(str, Enum):
    VersionsAutodeskBim360Document = "versions:autodesk.bim360:Document",
    VersionsAutodeskBim360File = "versions:autodesk.bim360:File",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .viewcontext_post_response_clash_group import ViewcontextPostResponse_clashGroup
    from .viewcontext_post_response_documents import ViewcontextPostResponse_documents

@dataclass
class ViewcontextPostResponse(Parsable):
    # A Clash Group.
    clash_group: Optional[list[ViewcontextPostResponse_clashGroup]] = None
    # The list of documents visible when the issue was created.
    documents: Optional[list[ViewcontextPostResponse_documents]] = None
    # The ID of the assigned clash group issue for retrieving the issue.
    issue_id: Optional[UUID] = None
    # The ID of the model set this assigned clash group issue is associated with.
    model_set_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ViewcontextPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ViewcontextPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ViewcontextPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .viewcontext_post_response_clash_group import ViewcontextPostResponse_clashGroup
        from .viewcontext_post_response_documents import ViewcontextPostResponse_documents

        from .viewcontext_post_response_clash_group import ViewcontextPostResponse_clashGroup
        from .viewcontext_post_response_documents import ViewcontextPostResponse_documents

        fields: dict[str, Callable[[Any], None]] = {
            "clashGroup": lambda n : setattr(self, 'clash_group', n.get_collection_of_object_values(ViewcontextPostResponse_clashGroup)),
            "documents": lambda n : setattr(self, 'documents', n.get_collection_of_object_values(ViewcontextPostResponse_documents)),
            "issueId": lambda n : setattr(self, 'issue_id', n.get_uuid_value()),
            "modelSetId": lambda n : setattr(self, 'model_set_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("clashGroup", self.clash_group)
        writer.write_collection_of_object_values("documents", self.documents)
        writer.write_uuid_value("issueId", self.issue_id)
        writer.write_uuid_value("modelSetId", self.model_set_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithWorkflowGetResponse_copyFilesOptions(Parsable):
    """
    (``Copy approved files`` in the UI) The configuration for copying approved files to a target folder when the review is complete.
    """
    # (``Allow the initiator to change the target folder`` in the UI)  Allows the initiator to change the target folder when creating a review.``true``: the initiator can choose a different target folder.``false``: the folder defined in the workflow is used.
    allow_override: Optional[bool] = None
    # (``All/Any files in the review have been approved`` in the UI)Specifies the condition under which approved files will be copied.Possible values:``ANY``: copy files if at least one file in the review is approved.``ALL``: copy files only if all files in the review are approved.
    condition: Optional[str] = None
    # (``Allow approvers to change whether or not markups are included`` in the UI)Controls whether approvers or admins can change the markup inclusion setting when starting the review.``true``: the markup setting is locked and cannot be changed.``false``: the setting can be changed during review setup.
    disable_override_markup_setting: Optional[bool] = None
    # Indicates whether approved files should be copied to a target folder after the review is complete.``true``: copy approved files to the target folder.``false``: do not copy approved files.
    enabled: Optional[bool] = None
    # (``Then copy approved files to`` in the UI)The URN of the target folder where approved files will be copied.
    folder_urn: Optional[str] = None
    # (``Include all published markups on approved files`` in the UI)Indicates whether published markups should be included when copying files.``true``: include all published (unarchived) markups from the source version.``false``: (default) do not include markups.
    include_markups: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithWorkflowGetResponse_copyFilesOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithWorkflowGetResponse_copyFilesOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithWorkflowGetResponse_copyFilesOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "allowOverride": lambda n : setattr(self, 'allow_override', n.get_bool_value()),
            "condition": lambda n : setattr(self, 'condition', n.get_str_value()),
            "disableOverrideMarkupSetting": lambda n : setattr(self, 'disable_override_markup_setting', n.get_bool_value()),
            "enabled": lambda n : setattr(self, 'enabled', n.get_bool_value()),
            "folderUrn": lambda n : setattr(self, 'folder_urn', n.get_str_value()),
            "includeMarkups": lambda n : setattr(self, 'include_markups', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("allowOverride", self.allow_override)
        writer.write_str_value("condition", self.condition)
        writer.write_bool_value("disableOverrideMarkupSetting", self.disable_override_markup_setting)
        writer.write_bool_value("enabled", self.enabled)
        writer.write_str_value("folderUrn", self.folder_urn)
        writer.write_bool_value("includeMarkups", self.include_markups)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .revisions_get_response_results_steps_tasks import RevisionsGetResponse_results_steps_tasks

@dataclass
class RevisionsGetResponse_results_steps(Parsable):
    # The date when the step was completed, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``. Note that this value will be ``null`` if not completed.
    completed_at: Optional[datetime.datetime] = None
    # The date when the step was created, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    created_at: Optional[datetime.datetime] = None
    # A way to specify a dynamic due date instead of the the regular dueDate field. When the step will start the due date will be calculated and assigned according to this field.
    days_to_respond: Optional[float] = None
    # The due date for this step, in the following format: YYYY-MM-DD (ISO 8601) in UTC.  For example, ``2025-01-27T12:09:24.198466Z``.
    due_date: Optional[str] = None
    # The revision number of the submittal item when this step was recorded.
    revision: Optional[int] = None
    # The date and time when the step began, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``. Note that this value will be ``null`` if not started.
    started_at: Optional[datetime.datetime] = None
    # The unique identifier of the step within the submittal item's workflow for this revision.
    step_id: Optional[UUID] = None
    # The order of the step in the workflow at this revision. The first step is ``1``.
    step_number: Optional[float] = None
    # A list of tasks related to a specific step within the workflow.
    tasks: Optional[list[RevisionsGetResponse_results_steps_tasks]] = None
    # The date when the step was last updated, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RevisionsGetResponse_results_steps:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RevisionsGetResponse_results_steps
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RevisionsGetResponse_results_steps()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .revisions_get_response_results_steps_tasks import RevisionsGetResponse_results_steps_tasks

        from .revisions_get_response_results_steps_tasks import RevisionsGetResponse_results_steps_tasks

        fields: dict[str, Callable[[Any], None]] = {
            "completedAt": lambda n : setattr(self, 'completed_at', n.get_datetime_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "daysToRespond": lambda n : setattr(self, 'days_to_respond', n.get_float_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "revision": lambda n : setattr(self, 'revision', n.get_int_value()),
            "startedAt": lambda n : setattr(self, 'started_at', n.get_datetime_value()),
            "stepId": lambda n : setattr(self, 'step_id', n.get_uuid_value()),
            "stepNumber": lambda n : setattr(self, 'step_number', n.get_float_value()),
            "tasks": lambda n : setattr(self, 'tasks', n.get_collection_of_object_values(RevisionsGetResponse_results_steps_tasks)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("completedAt", self.completed_at)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_float_value("daysToRespond", self.days_to_respond)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_int_value("revision", self.revision)
        writer.write_datetime_value("startedAt", self.started_at)
        writer.write_uuid_value("stepId", self.step_id)
        writer.write_float_value("stepNumber", self.step_number)
        writer.write_collection_of_object_values("tasks", self.tasks)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class PaymentsGetResponse_additionalCollaborators(Parsable):
    # The BIM360/ACC ID of the firm.
    company_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    company_uid: Optional[str] = None
    # The BIM360/ACC user ID of the contacts in the firm.
    contact_ids: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PaymentsGetResponse_additionalCollaborators:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PaymentsGetResponse_additionalCollaborators
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PaymentsGetResponse_additionalCollaborators()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "companyId": lambda n : setattr(self, 'company_id', n.get_str_value()),
            "companyUid": lambda n : setattr(self, 'company_uid', n.get_str_value()),
            "contactIds": lambda n : setattr(self, 'contact_ids', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("companyId", self.company_id)
        writer.write_str_value("companyUid", self.company_uid)
        writer.write_collection_of_primitive_values("contactIds", self.contact_ids)
    




from enum import Enum

class SubmittalsSummaryGetResponse_submittalGroups_submittalGroup(str, Enum):
    ACTIONANDINFORMATIONAL = "ACTION AND INFORMATIONAL",
    CLOSEOUTSUBMITTALS = "CLOSEOUT SUBMITTALS",
    FIELDQUALITYCONTROL = "FIELD QUALITY CONTROL",
    Mockups = "Mockups",
    QUALITYASSURANCE = "QUALITY ASSURANCE",
    TestsAndInspections = "Tests And Inspections",
    DIVISION01REQUIREMENTS = "DIVISION 01 REQUIREMENTS",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_upload_get_response_status import WithUploadGetResponse_status

@dataclass
class WithUploadGetResponse(Parsable):
    # The time when the upload was created, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    created_at: Optional[datetime.datetime] = None
    # The ID of the user who created the upload.
    created_by: Optional[str] = None
    # The name of the user who created the upload.
    created_by_name: Optional[str] = None
    # The ID of the upload.
    id: Optional[UUID] = None
    # The time when all the review sheets of the upload were published, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    published_at: Optional[datetime.datetime] = None
    # The ID of the user who published all the review sheets of the upload.
    published_by: Optional[str] = None
    # The name of the user who published all the review sheets of the upload.
    published_by_name: Optional[str] = None
    # The number of files that have been published by the upload.
    published_count: Optional[int] = None
    # The status of the upload. Possible values:- ``PENDING``: the uploaded files are waiting for to be processed.- ``PROCESSING``: the uploaded files are being processed.- ``IN_REVIEW``: the file upload process is complete. The sheets are ready for review. You can now call `GET review-sheets </en/docs/acc/v1/reference/http/sheets-review-sheets-GET/>`_, `PATCH review-sheets </en/docs/acc/v1/reference/http/sheets-review-sheets-PATCH/>`_, or `POST review-sheets:publish </en/docs/acc/v1/reference/http/sheets-review-sheetspublish-POST/>`_.- ``FAILED``: the file upload process failed. One of the final status of an upload.- ``UPDATING_VERSION_SET``: the target version set is being updated.- ``PUBLISHING``: the review sheets are being published.- ``PUBLISHED``: the review sheets have been published.
    status: Optional[WithUploadGetResponse_status] = None
    # The time when the upload was last updated, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    updated_at: Optional[datetime.datetime] = None
    # The ID of the user who last updated the upload.
    updated_by: Optional[str] = None
    # The name of the user who last updated the upload.
    updated_by_name: Optional[str] = None
    # The ID of the version set where the upload creates sheets to.
    version_set_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithUploadGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithUploadGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithUploadGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_upload_get_response_status import WithUploadGetResponse_status

        from .with_upload_get_response_status import WithUploadGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByName": lambda n : setattr(self, 'created_by_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "publishedAt": lambda n : setattr(self, 'published_at', n.get_datetime_value()),
            "publishedBy": lambda n : setattr(self, 'published_by', n.get_str_value()),
            "publishedByName": lambda n : setattr(self, 'published_by_name', n.get_str_value()),
            "publishedCount": lambda n : setattr(self, 'published_count', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithUploadGetResponse_status)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
            "versionSetId": lambda n : setattr(self, 'version_set_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByName", self.created_by_name)
        writer.write_uuid_value("id", self.id)
        writer.write_datetime_value("publishedAt", self.published_at)
        writer.write_str_value("publishedBy", self.published_by)
        writer.write_str_value("publishedByName", self.published_by_name)
        writer.write_int_value("publishedCount", self.published_count)
        writer.write_enum_value("status", self.status)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
        writer.write_uuid_value("versionSetId", self.version_set_id)
    




from enum import Enum

class FieldsGetResponse_type(str, Enum):
    Unknown = "Unknown",
    Boolean = "Boolean",
    Integer = "Integer",
    Double = "Double",
    Blob = "Blob",
    DbKey = "DbKey",
    String = "String",
    LocalizableString = "LocalizableString",
    DateTime = "DateTime",
    GeoLocation = "GeoLocation",
    Position = "Position",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .permissions_batch_delete_post_request_body_subject_type import PermissionsBatchDeletePostRequestBody_subjectType

@dataclass
class PermissionsBatchDeletePostRequestBody(Parsable):
    # The Autodesk ID of the user, role or company.
    autodesk_id: Optional[str] = None
    # The ID of the user, role, or company. To verify the subjectId of the user, role, or company, use `GET permissions </en/docs/bim360/v1/reference/http/document-management-projects-project_id-folders-folder_id-permissions-GET>`_.
    subject_id: Optional[UUID] = None
    # The type of subject.Possible values: ``USER``, ``COMPANY``, ``ROLE``
    subject_type: Optional[PermissionsBatchDeletePostRequestBody_subjectType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PermissionsBatchDeletePostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PermissionsBatchDeletePostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PermissionsBatchDeletePostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .permissions_batch_delete_post_request_body_subject_type import PermissionsBatchDeletePostRequestBody_subjectType

        from .permissions_batch_delete_post_request_body_subject_type import PermissionsBatchDeletePostRequestBody_subjectType

        fields: dict[str, Callable[[Any], None]] = {
            "autodeskId": lambda n : setattr(self, 'autodesk_id', n.get_str_value()),
            "subjectId": lambda n : setattr(self, 'subject_id', n.get_uuid_value()),
            "subjectType": lambda n : setattr(self, 'subject_type', n.get_enum_value(PermissionsBatchDeletePostRequestBody_subjectType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("autodeskId", self.autodesk_id)
        writer.write_uuid_value("subjectId", self.subject_id)
        writer.write_enum_value("subjectType", self.subject_type)
    




from enum import Enum

class GetAssociationTypeQueryParameterType(str, Enum):
    Contract = "Contract",
    MainContract = "MainContract",
    BudgetPayment = "BudgetPayment",
    CostPayment = "CostPayment",
    OCO = "OCO",
    SCO = "SCO",
    PCO = "PCO",
    RFQ = "RFQ",
    RCO = "RCO",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .manifest_get_response_seed_files_databases import ManifestGetResponse_seedFiles_databases
    from .manifest_get_response_seed_files_views import ManifestGetResponse_seedFiles_views

@dataclass
class ManifestGetResponse_seedFiles(Parsable):
    # databases.
    databases: Optional[list[ManifestGetResponse_seedFiles_databases]] = None
    # lineage ID.
    lineage_id: Optional[str] = None
    # lineage URN.
    lineage_urn: Optional[str] = None
    # file version URN.
    version_urn: Optional[str] = None
    # views.
    views: Optional[list[ManifestGetResponse_seedFiles_views]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ManifestGetResponse_seedFiles:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ManifestGetResponse_seedFiles
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ManifestGetResponse_seedFiles()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .manifest_get_response_seed_files_databases import ManifestGetResponse_seedFiles_databases
        from .manifest_get_response_seed_files_views import ManifestGetResponse_seedFiles_views

        from .manifest_get_response_seed_files_databases import ManifestGetResponse_seedFiles_databases
        from .manifest_get_response_seed_files_views import ManifestGetResponse_seedFiles_views

        fields: dict[str, Callable[[Any], None]] = {
            "databases": lambda n : setattr(self, 'databases', n.get_collection_of_object_values(ManifestGetResponse_seedFiles_databases)),
            "lineageId": lambda n : setattr(self, 'lineage_id', n.get_str_value()),
            "lineageUrn": lambda n : setattr(self, 'lineage_urn', n.get_str_value()),
            "versionUrn": lambda n : setattr(self, 'version_urn', n.get_str_value()),
            "views": lambda n : setattr(self, 'views', n.get_collection_of_object_values(ManifestGetResponse_seedFiles_views)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("databases", self.databases)
        writer.write_str_value("lineageId", self.lineage_id)
        writer.write_str_value("lineageUrn", self.lineage_urn)
        writer.write_str_value("versionUrn", self.version_urn)
        writer.write_collection_of_object_values("views", self.views)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .workflow_get_response_steps_group_review_type import WorkflowGetResponse_steps_groupReview_type

@dataclass
class WorkflowGetResponse_steps_groupReview(Parsable):
    """
    (``Reviewer Type`` in the UI) Defines whether multiple reviewers can participate in this step and how their responses are handled. It applies only to ``REVIEWER`` steps.
    """
    # Indicates whether group review is enabled for this step.``true``: multiple reviewers can participate in the step.``false``: (default) only a single reviewer is allowed.
    enabled: Optional[bool] = None
    # The minimum number of reviewers required for this step. This field is set automatically when the group review type is set to ``MINIMUM``. It is not independently configurable. Valid range: ``1–30``.
    min: Optional[int] = None
    # (Displayed under ``More options`` in the UI) Specifies the group review rule for this step.Possible values:- ``ALL``: every reviewer assigned to the step must submit a response (up to 30).- ``MINIMUM``: only a specified number of reviewers must respond, as defined in ``min``.
    type: Optional[WorkflowGetResponse_steps_groupReview_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowGetResponse_steps_groupReview:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowGetResponse_steps_groupReview
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowGetResponse_steps_groupReview()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflow_get_response_steps_group_review_type import WorkflowGetResponse_steps_groupReview_type

        from .workflow_get_response_steps_group_review_type import WorkflowGetResponse_steps_groupReview_type

        fields: dict[str, Callable[[Any], None]] = {
            "enabled": lambda n : setattr(self, 'enabled', n.get_bool_value()),
            "min": lambda n : setattr(self, 'min', n.get_int_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(WorkflowGetResponse_steps_groupReview_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("enabled", self.enabled)
        writer.write_int_value("min", self.min)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_diff_item_request_builder import WithDiffItemRequestBuilder

class DiffsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/index/v2/projects/{projectId}/diffs
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new DiffsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/index/v2/projects/{projectId}/diffs", path_parameters)
    
    def by_diff_id(self,diff_id: str) -> WithDiffItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.index.v2.projects.item.diffs.item collection
        param diff_id: The diff ID.
        Returns: WithDiffItemRequestBuilder
        """
        if diff_id is None:
            raise TypeError("diff_id cannot be null.")
        from .item.with_diff_item_request_builder import WithDiffItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["diffId"] = diff_id
        return WithDiffItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.with_account_item_request_builder import WithAccountItemRequestBuilder

class AccountsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /data-connector/v1/accounts
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new AccountsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/data-connector/v1/accounts", path_parameters)
    
    def by_account_id(self,account_id: UUID) -> WithAccountItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.dataConnector.v1.accounts.item collection
        param account_id: The account ID. You can derive it from your hub ID if necessary: Use `GET hubs </en/docs/data/v2/reference/http/hubs-GET/>`_ in the Data Management API to retrieve your hub ID. Remove the initial ``b.`` from the hub ID to get your account ID.
        Returns: WithAccountItemRequestBuilder
        """
        if account_id is None:
            raise TypeError("account_id cannot be null.")
        from .item.with_account_item_request_builder import WithAccountItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["accountId"] = account_id
        return WithAccountItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_item_id_transition_post_response_ball_in_court_type import WithItemIdTransitionPostResponse_ballInCourtType
    from .with_item_id_transition_post_response_manager_type import WithItemIdTransitionPostResponse_managerType
    from .with_item_id_transition_post_response_permitted_actions import WithItemIdTransitionPostResponse_permittedActions
    from .with_item_id_transition_post_response_priority import WithItemIdTransitionPostResponse_priority
    from .with_item_id_transition_post_response_revisions_folders_urns import WithItemIdTransitionPostResponse_revisionsFoldersUrns
    from .with_item_id_transition_post_response_state_id import WithItemIdTransitionPostResponse_stateId
    from .with_item_id_transition_post_response_status_id import WithItemIdTransitionPostResponse_statusId
    from .with_item_id_transition_post_response_subcontractor_type import WithItemIdTransitionPostResponse_subcontractorType
    from .with_item_id_transition_post_response_watchers import WithItemIdTransitionPostResponse_watchers

@dataclass
class WithItemIdTransitionPostResponse(Parsable):
    # The member group IDs of the companies currently assigned to the submittal item at this stage of the workflow.
    ball_in_court_companies: Optional[list[str]] = None
    # The member group IDs of user roles that are currently assigned to the submittal item at this stage of the workflow.
    ball_in_court_roles: Optional[list[str]] = None
    # 'The type of submittal role assigned to the user currently assigned to the submittal item.Possible values: ``reviewer``, ``manager``, ``subcontractor``.'
    ball_in_court_type: Optional[WithItemIdTransitionPostResponse_ballInCourtType] = None
    # The Autodesk IDs of users who are currently assigned to the submittal item at this stage of the workflow.
    ball_in_court_users: Optional[list[str]] = None
    # The date and time when the submittal item was originally created, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the submittal item.
    created_by: Optional[str] = None
    # ``customIdentifier`` and ``customIdentifierHumanReadable`` relate to the ``Number`` column in the UI. Submittal managers assign custom numbers to items (manually or automatically). Custom numbers are configured either in global numbering format: <global number>, or in spec section numbering format: <spec ID>-<sequential number>.For projects with a global numbering format, both ``customIdentifier`` and ``customIdentifierHumanReadable`` represent the global number.For projects with a spec section numbering format (<spec ID>-<sequential number>), ``customIdentifier`` represents the sequential number. For example, for a full number of ``033100-01``, ``customIdentifier`` is ``01``.Note that for unnumbered items ``customIdentifier`` is ``null``.For more information on custom numbering, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Custom_Numbering>`_.
    custom_identifier: Optional[str] = None
    # ``customIdentifierHumanReadable`` and ``customIdentifier`` relate to the ``Number`` column in the UI. Submittal managers assign custom numbers to items (manually or automatically). Custom numbers are configured either in global numbering format: <global number>, or in spec section numbering format: <spec ID>-<sequential number>.For projects with a global numbering format, both ``customIdentifier`` and ``customIdentifierHumanReadable`` represent the global number.For projects with a spec section numbering format (<spec ID>-<sequential number>), ``customIdentifierHumanReadable`` represents the full number - both the spec ID and the sequential number. For example, for a full number of ``033100-01``, ``customIdentifierHumanReadable`` is ``033100-01``.Note that for unnumbered items ``customIdentifierHumanReadable`` is ``Unspecified``. For spec section numbering it also includes the spec ID. For example, ``033100-Unspecified``.For more information on custom numbering, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Custom_Numbering>`_.
    custom_identifier_human_readable: Optional[str] = None
    # The description of the submittal item.
    description: Optional[str] = None
    # The due date for the submittal item, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    due_date: Optional[str] = None
    # The URN of the folder that contains the attachments associated with the submittal items.
    folder_urn: Optional[str] = None
    # The internal, globally unique identifier (UUID) for the submittal item.
    id: Optional[UUID] = None
    # The unique ID assigned to the submittal item within the UI. This ID is system-generated and serves as a reference for users interacting with submittal items through the UI. For example, ``111``.
    identifier: Optional[int] = None
    # The duration (in days) from the approval of the submittal to delivery of materials or products to the construction site.
    lead_time: Optional[int] = None
    # The ID that was assigned to the manager of the submittal item.To determine the type of the manager (user, role, or company), refer to the manager type (``managerType``) attribute.In order to get more info about the manager, use:- `GET projects/users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_ to verify the actual name of the user in case the typs is a ``user`` (1).- `GET companies </en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_ to determine the name of the company in case the typs is a ``company`` (2).Note that we do not currently support verifying names of roles.
    manager: Optional[str] = None
    # The date by which the manager is expected to prepare the submittal item for review, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``. This corresponds to the ``mgr-1`` state ``Open (Submitted)``.
    manager_due_date: Optional[str] = None
    # The type of manager associated with the submittal item.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    manager_type: Optional[WithItemIdTransitionPostResponse_managerType] = None
    # The ID of the package associated with the submittal item.
    package_id: Optional[UUID] = None
    # The package identifier as displayed in the UI.
    package_identifier: Optional[str] = None
    # The identifier of the submittal spec associated with the package. This value corresponds to the ``Spec section`` displayed in the UI, such as ``1 - Cement``.
    package_spec_identifier: Optional[str] = None
    # The title of the package associated with the submittal item.
    package_title: Optional[str] = None
    # The list of actions the user is allowed to perform on the submittal item.
    permitted_actions: Optional[list[WithItemIdTransitionPostResponse_permittedActions]] = None
    # The priority of the submittal item.Possible values:- ``Low``- ``Normal``- ``High``
    priority: Optional[WithItemIdTransitionPostResponse_priority] = None
    # The Autodesk ID of the user who published the submittal item.
    published_by: Optional[str] = None
    # The date when the manager closed and distributed the submittal item, in the following format: YYYY-MM-DD (ISO 8601) in UTC. For example, ``2018-02-15``.
    published_date: Optional[datetime.datetime] = None
    # The date and time when the submittal item transitioned from the ``rev`` state ``(Open - In Review)`` to the ``mgr-2`` state ``(Close and distribute)``, formatted as YYYY-MM-DD (ISO 8601) in UTC. For example, ``2022-03-02T12:09:24Z``.
    received_from_review: Optional[datetime.datetime] = None
    # The date when the submittal was received back from the subcontractor after review, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``. This corresponds to transition to ``mgr-1`` state ``Open (Submitted)``.
    received_from_submitter: Optional[datetime.datetime] = None
    # The date by which approval for the submittal is required, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    required_approval_date: Optional[str] = None
    # The date by which the Responsible Contractor must submit the submittal to the submittal manager, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    required_date: Optional[str] = None
    # The date when the materials are expected to arrive on the construction site, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    required_on_job_date: Optional[str] = None
    # The date and time when the response was added, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    responded_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that gave the response to the submittal item.
    responded_by: Optional[str] = None
    # The body of the response comment, containing feedback or instructions related to the submittal item.
    response_comment: Optional[str] = None
    # The ID of the response associated with the submittal item, linking to the specific feedback or action taken.
    response_id: Optional[str] = None
    # The revision number of the submittal item, indicating the version of the item in the submittal workflow. For example, ``1`` for the initial submission or ``2`` for the first revision.
    revision: Optional[int] = None
    # An object containing URNs that represent folders associated with the revisions of the submittal item. These URNs can be used to access and identify specific folders related to submittal item revisions within the system.
    revisions_folders_urns: Optional[WithItemIdTransitionPostResponse_revisionsFoldersUrns] = None
    # The date and time when the submittal item transitioned to the ``rev`` state ``(Open - In review)``, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    sent_to_review: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who transitioned the item to the ``rev`` state ``(Open - In review)``.
    sent_to_review_by: Optional[str] = None
    # The date and time when the submittal was sent to the subcontractor for review, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``. This corresponds to transition to the ``sbc-1`` state.
    sent_to_submitter: Optional[datetime.datetime] = None
    # The unique identifier (UUID) of the spec assigned to the submittal item.
    spec_id: Optional[UUID] = None
    # The identifier of the spec section that is associated with the submittal item. The identifier is assigned to the spec section in the UI.
    spec_identifier: Optional[str] = None
    # The title of the spec associated with the submittal item.
    spec_title: Optional[str] = None
    # The current state of the submittal item after the transition.Possible values:``sbc-1`` (Waiting for Submission) - Assigned to the Responsible Contractor, who needs to submit the submittal to the Manager.``mgr-1`` (Open - Submitted) - Assigned to the Manager, who needs to prepare the submittal item for review.``rev`` (Open - In Review) - Under review by the reviewers defined in the submittal item's review workflow.``mgr-2`` (Open - Reviewed) - The review is complete, and the submittal is returned to the Manager, who needs to set the final response and close the submittal.``sbc-2``: (Closed) - The submittal has been closed and assigned to the Responsible Contractor.``void`` (Voided) - The submittal item has been voided.``draft`` (Draft) - Assigned to the Manager, who must send the submittal item to the Responsible Contractor.
    state_id: Optional[WithItemIdTransitionPostResponse_stateId] = None
    # The status of the submittal item.Possible values: ``1`` - (Required), ``2`` - (Open), ``3`` - (Closed), ``4`` - (Void), ``5`` - (Empty), ``6`` - (Draft).To retrieve the full list of possible statuses, call `GET metadata </en/docs/acc/v1/reference/http/submittals-metadata-GET/>`_.
    status_id: Optional[WithItemIdTransitionPostResponse_statusId] = None
    # The ID that was assigned to the subcontractor for the submittal item. If a non-manager user created the submittal item and chose a manager, they are automatically assigned as the ``subcontractor`` of the submittal item.In order to get more info about the subcontractor, use:- `GET projects/users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_ to verify the actual name of the user in case the typs is a ``user`` (1).- `GET companies </en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_ to determine the name of the company in case the typs is a ``company`` (2).Note that we do not currently support verifying names of roles.
    subcontractor: Optional[str] = None
    # The type of subcontractor associated with the submittal item.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    subcontractor_type: Optional[WithItemIdTransitionPostResponse_subcontractorType] = None
    # The Autodesk ID of the user who submitted the submittal item. This is the user who transitioned the item to the manager.
    submitted_by: Optional[str] = None
    # The date by which the subcontractor is expected to submit the submittal to the manager, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``. This corresponds to the ``sbc-1`` state ``Waiting for submission``.
    submitter_due_date: Optional[str] = None
    # The sub-spec section associated with the submittal item, providing additional categorization within the main spec.
    subsection: Optional[str] = None
    # The title of the submittal item.
    title: Optional[str] = None
    # The ID representing the type of submittal item.
    type_id: Optional[str] = None
    # The time and date when the submittal item was last updated, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the submittal item.
    updated_by: Optional[str] = None
    # A list of project watchers, who can be individual users, roles, or companies.
    watchers: Optional[list[WithItemIdTransitionPostResponse_watchers]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithItemIdTransitionPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithItemIdTransitionPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithItemIdTransitionPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_item_id_transition_post_response_ball_in_court_type import WithItemIdTransitionPostResponse_ballInCourtType
        from .with_item_id_transition_post_response_manager_type import WithItemIdTransitionPostResponse_managerType
        from .with_item_id_transition_post_response_permitted_actions import WithItemIdTransitionPostResponse_permittedActions
        from .with_item_id_transition_post_response_priority import WithItemIdTransitionPostResponse_priority
        from .with_item_id_transition_post_response_revisions_folders_urns import WithItemIdTransitionPostResponse_revisionsFoldersUrns
        from .with_item_id_transition_post_response_state_id import WithItemIdTransitionPostResponse_stateId
        from .with_item_id_transition_post_response_status_id import WithItemIdTransitionPostResponse_statusId
        from .with_item_id_transition_post_response_subcontractor_type import WithItemIdTransitionPostResponse_subcontractorType
        from .with_item_id_transition_post_response_watchers import WithItemIdTransitionPostResponse_watchers

        from .with_item_id_transition_post_response_ball_in_court_type import WithItemIdTransitionPostResponse_ballInCourtType
        from .with_item_id_transition_post_response_manager_type import WithItemIdTransitionPostResponse_managerType
        from .with_item_id_transition_post_response_permitted_actions import WithItemIdTransitionPostResponse_permittedActions
        from .with_item_id_transition_post_response_priority import WithItemIdTransitionPostResponse_priority
        from .with_item_id_transition_post_response_revisions_folders_urns import WithItemIdTransitionPostResponse_revisionsFoldersUrns
        from .with_item_id_transition_post_response_state_id import WithItemIdTransitionPostResponse_stateId
        from .with_item_id_transition_post_response_status_id import WithItemIdTransitionPostResponse_statusId
        from .with_item_id_transition_post_response_subcontractor_type import WithItemIdTransitionPostResponse_subcontractorType
        from .with_item_id_transition_post_response_watchers import WithItemIdTransitionPostResponse_watchers

        fields: dict[str, Callable[[Any], None]] = {
            "ballInCourtCompanies": lambda n : setattr(self, 'ball_in_court_companies', n.get_collection_of_primitive_values(str)),
            "ballInCourtRoles": lambda n : setattr(self, 'ball_in_court_roles', n.get_collection_of_primitive_values(str)),
            "ballInCourtType": lambda n : setattr(self, 'ball_in_court_type', n.get_enum_value(WithItemIdTransitionPostResponse_ballInCourtType)),
            "ballInCourtUsers": lambda n : setattr(self, 'ball_in_court_users', n.get_collection_of_primitive_values(str)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "customIdentifier": lambda n : setattr(self, 'custom_identifier', n.get_str_value()),
            "customIdentifierHumanReadable": lambda n : setattr(self, 'custom_identifier_human_readable', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "folderUrn": lambda n : setattr(self, 'folder_urn', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "identifier": lambda n : setattr(self, 'identifier', n.get_int_value()),
            "leadTime": lambda n : setattr(self, 'lead_time', n.get_int_value()),
            "manager": lambda n : setattr(self, 'manager', n.get_str_value()),
            "managerDueDate": lambda n : setattr(self, 'manager_due_date', n.get_str_value()),
            "managerType": lambda n : setattr(self, 'manager_type', n.get_enum_value(WithItemIdTransitionPostResponse_managerType)),
            "packageId": lambda n : setattr(self, 'package_id', n.get_uuid_value()),
            "packageIdentifier": lambda n : setattr(self, 'package_identifier', n.get_str_value()),
            "packageSpecIdentifier": lambda n : setattr(self, 'package_spec_identifier', n.get_str_value()),
            "packageTitle": lambda n : setattr(self, 'package_title', n.get_str_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(WithItemIdTransitionPostResponse_permittedActions)),
            "priority": lambda n : setattr(self, 'priority', n.get_enum_value(WithItemIdTransitionPostResponse_priority)),
            "publishedBy": lambda n : setattr(self, 'published_by', n.get_str_value()),
            "publishedDate": lambda n : setattr(self, 'published_date', n.get_datetime_value()),
            "receivedFromReview": lambda n : setattr(self, 'received_from_review', n.get_datetime_value()),
            "receivedFromSubmitter": lambda n : setattr(self, 'received_from_submitter', n.get_datetime_value()),
            "requiredApprovalDate": lambda n : setattr(self, 'required_approval_date', n.get_str_value()),
            "requiredDate": lambda n : setattr(self, 'required_date', n.get_str_value()),
            "requiredOnJobDate": lambda n : setattr(self, 'required_on_job_date', n.get_str_value()),
            "respondedAt": lambda n : setattr(self, 'responded_at', n.get_datetime_value()),
            "respondedBy": lambda n : setattr(self, 'responded_by', n.get_str_value()),
            "responseComment": lambda n : setattr(self, 'response_comment', n.get_str_value()),
            "responseId": lambda n : setattr(self, 'response_id', n.get_str_value()),
            "revision": lambda n : setattr(self, 'revision', n.get_int_value()),
            "revisionsFoldersUrns": lambda n : setattr(self, 'revisions_folders_urns', n.get_object_value(WithItemIdTransitionPostResponse_revisionsFoldersUrns)),
            "sentToReview": lambda n : setattr(self, 'sent_to_review', n.get_datetime_value()),
            "sentToReviewBy": lambda n : setattr(self, 'sent_to_review_by', n.get_str_value()),
            "sentToSubmitter": lambda n : setattr(self, 'sent_to_submitter', n.get_datetime_value()),
            "specId": lambda n : setattr(self, 'spec_id', n.get_uuid_value()),
            "specIdentifier": lambda n : setattr(self, 'spec_identifier', n.get_str_value()),
            "specTitle": lambda n : setattr(self, 'spec_title', n.get_str_value()),
            "stateId": lambda n : setattr(self, 'state_id', n.get_enum_value(WithItemIdTransitionPostResponse_stateId)),
            "statusId": lambda n : setattr(self, 'status_id', n.get_enum_value(WithItemIdTransitionPostResponse_statusId)),
            "subcontractor": lambda n : setattr(self, 'subcontractor', n.get_str_value()),
            "subcontractorType": lambda n : setattr(self, 'subcontractor_type', n.get_enum_value(WithItemIdTransitionPostResponse_subcontractorType)),
            "submittedBy": lambda n : setattr(self, 'submitted_by', n.get_str_value()),
            "submitterDueDate": lambda n : setattr(self, 'submitter_due_date', n.get_str_value()),
            "subsection": lambda n : setattr(self, 'subsection', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "typeId": lambda n : setattr(self, 'type_id', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(WithItemIdTransitionPostResponse_watchers)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("ballInCourtCompanies", self.ball_in_court_companies)
        writer.write_collection_of_primitive_values("ballInCourtRoles", self.ball_in_court_roles)
        writer.write_enum_value("ballInCourtType", self.ball_in_court_type)
        writer.write_collection_of_primitive_values("ballInCourtUsers", self.ball_in_court_users)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("customIdentifier", self.custom_identifier)
        writer.write_str_value("customIdentifierHumanReadable", self.custom_identifier_human_readable)
        writer.write_str_value("description", self.description)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_str_value("folderUrn", self.folder_urn)
        writer.write_uuid_value("id", self.id)
        writer.write_int_value("identifier", self.identifier)
        writer.write_int_value("leadTime", self.lead_time)
        writer.write_str_value("manager", self.manager)
        writer.write_str_value("managerDueDate", self.manager_due_date)
        writer.write_enum_value("managerType", self.manager_type)
        writer.write_uuid_value("packageId", self.package_id)
        writer.write_str_value("packageIdentifier", self.package_identifier)
        writer.write_str_value("packageSpecIdentifier", self.package_spec_identifier)
        writer.write_str_value("packageTitle", self.package_title)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_enum_value("priority", self.priority)
        writer.write_str_value("publishedBy", self.published_by)
        writer.write_datetime_value("publishedDate", self.published_date)
        writer.write_datetime_value("receivedFromReview", self.received_from_review)
        writer.write_datetime_value("receivedFromSubmitter", self.received_from_submitter)
        writer.write_str_value("requiredApprovalDate", self.required_approval_date)
        writer.write_str_value("requiredDate", self.required_date)
        writer.write_str_value("requiredOnJobDate", self.required_on_job_date)
        writer.write_datetime_value("respondedAt", self.responded_at)
        writer.write_str_value("respondedBy", self.responded_by)
        writer.write_str_value("responseComment", self.response_comment)
        writer.write_str_value("responseId", self.response_id)
        writer.write_int_value("revision", self.revision)
        writer.write_object_value("revisionsFoldersUrns", self.revisions_folders_urns)
        writer.write_datetime_value("sentToReview", self.sent_to_review)
        writer.write_str_value("sentToReviewBy", self.sent_to_review_by)
        writer.write_datetime_value("sentToSubmitter", self.sent_to_submitter)
        writer.write_uuid_value("specId", self.spec_id)
        writer.write_str_value("specIdentifier", self.spec_identifier)
        writer.write_str_value("specTitle", self.spec_title)
        writer.write_enum_value("stateId", self.state_id)
        writer.write_enum_value("statusId", self.status_id)
        writer.write_str_value("subcontractor", self.subcontractor)
        writer.write_enum_value("subcontractorType", self.subcontractor_type)
        writer.write_str_value("submittedBy", self.submitted_by)
        writer.write_str_value("submitterDueDate", self.submitter_due_date)
        writer.write_str_value("subsection", self.subsection)
        writer.write_str_value("title", self.title)
        writer.write_str_value("typeId", self.type_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_collection_of_object_values("watchers", self.watchers)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .templates_get_response_results_watchers_user_type import TemplatesGetResponse_results_watchers_userType

@dataclass
class TemplatesGetResponse_results_watchers(Parsable):
    # The Autodesk ID of the watcher. The watcher can be a user (``autodeskId``), role (``memberGroupId``), or company (``memberGroupId``).To find details about users, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_, to find details about companies, call `GET companies <en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_. Note that we do not currently support finding details about roles for a project.
    id: Optional[str] = None
    # The type of watcher assigned to the submittal item.Possible values:- ``1`` (user)- ``2`` (company)- ``3`` (role)
    user_type: Optional[TemplatesGetResponse_results_watchers_userType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TemplatesGetResponse_results_watchers:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TemplatesGetResponse_results_watchers
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TemplatesGetResponse_results_watchers()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .templates_get_response_results_watchers_user_type import TemplatesGetResponse_results_watchers_userType

        from .templates_get_response_results_watchers_user_type import TemplatesGetResponse_results_watchers_userType

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "userType": lambda n : setattr(self, 'user_type', n.get_object_value(TemplatesGetResponse_results_watchers_userType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_object_value("userType", self.user_type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .reviews_post_request_body_workflow_options_steps_candidates import ReviewsPostRequestBody_workflowOptions_steps_candidates

@dataclass
class ReviewsPostRequestBody_workflowOptions_steps(Parsable):
    # (Displayed in the UI when selecting Reviewers/Approvers for a step) Defines the users, roles, or companies to specify for this step of the approval workflow. At least one of the fields (``users``, ``roles``, or ``companies``) must be provided.
    candidates: Optional[ReviewsPostRequestBody_workflowOptions_steps_candidates] = None
    # The ID of the step in the approval workflow. Use this value to set the candidates for the step when creating the review. Retrieve step IDs by calling `GET workflows </en/docs/acc/v1/reference/http/reviews-workflows-GET/>`_.
    id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewsPostRequestBody_workflowOptions_steps:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewsPostRequestBody_workflowOptions_steps
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewsPostRequestBody_workflowOptions_steps()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .reviews_post_request_body_workflow_options_steps_candidates import ReviewsPostRequestBody_workflowOptions_steps_candidates

        from .reviews_post_request_body_workflow_options_steps_candidates import ReviewsPostRequestBody_workflowOptions_steps_candidates

        fields: dict[str, Callable[[Any], None]] = {
            "candidates": lambda n : setattr(self, 'candidates', n.get_object_value(ReviewsPostRequestBody_workflowOptions_steps_candidates)),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("candidates", self.candidates)
        writer.write_str_value("id", self.id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .reviews_post_request_body_workflow_options_copy_files_options import ReviewsPostRequestBody_workflowOptions_copyFilesOptions
    from .reviews_post_request_body_workflow_options_steps import ReviewsPostRequestBody_workflowOptions_steps

@dataclass
class ReviewsPostRequestBody_workflowOptions(Parsable):
    # (``Copy approved files`` in the UI) Option for copying approved files when the review is complete.
    copy_files_options: Optional[ReviewsPostRequestBody_workflowOptions_copyFilesOptions] = None
    # Assigns participants to the steps defined in the approval workflow. If omitted, the review inherits default candidates from the workflow. If provided, your assignments replace the defaults for that step. Multiple ``REVIEWER`` steps run sequentially in the order returned by `GET workflows </en/docs/acc/v1/reference/http/reviews-workflows-GET/>`_.
    steps: Optional[list[ReviewsPostRequestBody_workflowOptions_steps]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewsPostRequestBody_workflowOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewsPostRequestBody_workflowOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewsPostRequestBody_workflowOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .reviews_post_request_body_workflow_options_copy_files_options import ReviewsPostRequestBody_workflowOptions_copyFilesOptions
        from .reviews_post_request_body_workflow_options_steps import ReviewsPostRequestBody_workflowOptions_steps

        from .reviews_post_request_body_workflow_options_copy_files_options import ReviewsPostRequestBody_workflowOptions_copyFilesOptions
        from .reviews_post_request_body_workflow_options_steps import ReviewsPostRequestBody_workflowOptions_steps

        fields: dict[str, Callable[[Any], None]] = {
            "copyFilesOptions": lambda n : setattr(self, 'copy_files_options', n.get_object_value(ReviewsPostRequestBody_workflowOptions_copyFilesOptions)),
            "steps": lambda n : setattr(self, 'steps', n.get_collection_of_object_values(ReviewsPostRequestBody_workflowOptions_steps)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("copyFilesOptions", self.copy_files_options)
        writer.write_collection_of_object_values("steps", self.steps)
    




from enum import Enum

class ValuesBatchUpdatePutResponse_customValues_toggleVal(str, Enum):
    Yes = "Yes",
    No = "No",
    False_ = "False",
    True_ = "True",
    Minus = "Minus",
    Plus = "Plus",
    Fail = "Fail",
    Pass_ = "Pass",
    NA = "NA",




from enum import Enum

class ActionsPostRequestBody_associationType(str, Enum):
    FormInstance = "FormInstance",
    OCO = "OCO",
    PCO = "PCO",
    RCO = "RCO",
    RFQ = "RFQ",
    SCO = "SCO",
    Expense = "Expense",
    Contract = "Contract",
    CostPayment = "CostPayment",
    BudgetPayment = "BudgetPayment",
    BudgetTransfer = "BudgetTransfer",
    MainContract = "MainContract",
    DistributionItem = "DistributionItem",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_transmittal_get_response_display_recipients import WithTransmittalGetResponse_displayRecipients
    from .with_transmittal_get_response_external_members import WithTransmittalGetResponse_externalMembers
    from .with_transmittal_get_response_packed_status import WithTransmittalGetResponse_packedStatus
    from .with_transmittal_get_response_recipients import WithTransmittalGetResponse_recipients
    from .with_transmittal_get_response_sent_by import WithTransmittalGetResponse_sentBy
    from .with_transmittal_get_response_status import WithTransmittalGetResponse_status

@dataclass
class WithTransmittalGetResponse(Parsable):
    # The date and time when the transmittal was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # Specifies how much recipient information each recipient can see.- ``ALL`` – All recipients can view the full recipient list.- ``LIMITED`` – Each recipient can view only their own recipient information.Project Admins and the sender always see the full list.Possible values: ``ALL``, ``LIMITED``
    display_recipients: Optional[WithTransmittalGetResponse_displayRecipients] = None
    # The total number of documents included in the transmittal.
    documents_count: Optional[int] = None
    # The list of external recipients who are not members of the project.
    external_members: Optional[list[WithTransmittalGetResponse_externalMembers]] = None
    # The unique identifier of the transmittal.
    id: Optional[UUID] = None
    # An optional message included with the transmittal.
    message: Optional[str] = None
    # Indicates the progress of packaging transmittal files into a ZIP archive.Possible values: ``SUCCESS``, ``PARTIAL_SUCCESS``, ``FAILED``, ``PROCESSING``, ``EXPIRED``, ``NOT_ALLOWED``
    packed_status: Optional[WithTransmittalGetResponse_packedStatus] = None
    # The list of recipients included in the transmittal, grouped by user, company, and role.For more information on how to add recipients to a transmittal, see the `Create Transmittals <https://help.autodesk.com/view/BUILD/ENU/?guid=Create_Transmittal&p=DOCS>`_ documentation.
    recipients: Optional[list[WithTransmittalGetResponse_recipients]] = None
    # Information about the user who created and sent the transmittal.
    sent_by: Optional[WithTransmittalGetResponse_sentBy] = None
    # A project-specific number automatically assigned when the transmittal is first submitted. It identifies the transmittal within the project and reflects the order in which it was created.
    sequence_id: Optional[str] = None
    # The current processing state of the transmittal.- ``SENDING`` – The transmittal is being processed or packaged; some fields may be temporarily empty. The ``recipients`` and ``externalMembers`` fields may be temporarily empty.- ``COMPLETED`` – The transmittal has been successfully issued and all data is available.- ``FAILED`` – The transmittal failed to process or send.Possible values: ``SENDING``, ``COMPLETED``, ``FAILED``.
    status: Optional[WithTransmittalGetResponse_status] = None
    # The title of the transmittal.
    title: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTransmittalGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTransmittalGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTransmittalGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_transmittal_get_response_display_recipients import WithTransmittalGetResponse_displayRecipients
        from .with_transmittal_get_response_external_members import WithTransmittalGetResponse_externalMembers
        from .with_transmittal_get_response_packed_status import WithTransmittalGetResponse_packedStatus
        from .with_transmittal_get_response_recipients import WithTransmittalGetResponse_recipients
        from .with_transmittal_get_response_sent_by import WithTransmittalGetResponse_sentBy
        from .with_transmittal_get_response_status import WithTransmittalGetResponse_status

        from .with_transmittal_get_response_display_recipients import WithTransmittalGetResponse_displayRecipients
        from .with_transmittal_get_response_external_members import WithTransmittalGetResponse_externalMembers
        from .with_transmittal_get_response_packed_status import WithTransmittalGetResponse_packedStatus
        from .with_transmittal_get_response_recipients import WithTransmittalGetResponse_recipients
        from .with_transmittal_get_response_sent_by import WithTransmittalGetResponse_sentBy
        from .with_transmittal_get_response_status import WithTransmittalGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "displayRecipients": lambda n : setattr(self, 'display_recipients', n.get_enum_value(WithTransmittalGetResponse_displayRecipients)),
            "documentsCount": lambda n : setattr(self, 'documents_count', n.get_int_value()),
            "externalMembers": lambda n : setattr(self, 'external_members', n.get_collection_of_object_values(WithTransmittalGetResponse_externalMembers)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "message": lambda n : setattr(self, 'message', n.get_str_value()),
            "packedStatus": lambda n : setattr(self, 'packed_status', n.get_enum_value(WithTransmittalGetResponse_packedStatus)),
            "recipients": lambda n : setattr(self, 'recipients', n.get_collection_of_object_values(WithTransmittalGetResponse_recipients)),
            "sentBy": lambda n : setattr(self, 'sent_by', n.get_object_value(WithTransmittalGetResponse_sentBy)),
            "sequenceId": lambda n : setattr(self, 'sequence_id', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithTransmittalGetResponse_status)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_enum_value("displayRecipients", self.display_recipients)
        writer.write_int_value("documentsCount", self.documents_count)
        writer.write_collection_of_object_values("externalMembers", self.external_members)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("message", self.message)
        writer.write_enum_value("packedStatus", self.packed_status)
        writer.write_collection_of_object_values("recipients", self.recipients)
        writer.write_object_value("sentBy", self.sent_by)
        writer.write_str_value("sequenceId", self.sequence_id)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("title", self.title)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .workflow_get_response_additional_options import WorkflowGetResponse_additionalOptions
    from .workflow_get_response_approval_status_options import WorkflowGetResponse_approvalStatusOptions
    from .workflow_get_response_attached_attributes import WorkflowGetResponse_attachedAttributes
    from .workflow_get_response_copy_files_options import WorkflowGetResponse_copyFilesOptions
    from .workflow_get_response_steps import WorkflowGetResponse_steps
    from .workflow_get_response_update_attributes_options import WorkflowGetResponse_updateAttributesOptions

@dataclass
class WorkflowGetResponse(Parsable):
    # Workflow-level settings that control whether the initiator can modify certain fields when starting a review.
    additional_options: Optional[WorkflowGetResponse_additionalOptions] = None
    # A list of file review status options to the workflow, which contains two built in options returned by the system.
    approval_status_options: Optional[list[WorkflowGetResponse_approvalStatusOptions]] = None
    # (``Update Attributes`` in the UI)The list of attributes added in the ``Update Attributes`` action.These attributes will be applied to the approved files in the target folder, or optionally also in the source folder depending on the configuration.
    attached_attributes: Optional[list[WorkflowGetResponse_attachedAttributes]] = None
    # (``Copy approved files`` in the UI) The configuration for copying approved files to a target folder when the review is complete.
    copy_files_options: Optional[WorkflowGetResponse_copyFilesOptions] = None
    # A description of the workflow.Max length: 4096
    description: Optional[str] = None
    # The ID of the workflow.
    id: Optional[UUID] = None
    # The name of the workflow. It must be unique within the project.Max length: 255
    name: Optional[str] = None
    # A custom note associated with the workflow. Visible to all reviewers during the review process.Max length: 4096
    notes: Optional[str] = None
    # A list of steps specify the details for each step in the workflow.
    steps: Optional[list[WorkflowGetResponse_steps]] = None
    # The configuration for applying attribute updates when a review is completed.This applies only if the workflow includes a file copy action and the ``Update Attributes`` action is enabled.
    update_attributes_options: Optional[WorkflowGetResponse_updateAttributesOptions] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflow_get_response_additional_options import WorkflowGetResponse_additionalOptions
        from .workflow_get_response_approval_status_options import WorkflowGetResponse_approvalStatusOptions
        from .workflow_get_response_attached_attributes import WorkflowGetResponse_attachedAttributes
        from .workflow_get_response_copy_files_options import WorkflowGetResponse_copyFilesOptions
        from .workflow_get_response_steps import WorkflowGetResponse_steps
        from .workflow_get_response_update_attributes_options import WorkflowGetResponse_updateAttributesOptions

        from .workflow_get_response_additional_options import WorkflowGetResponse_additionalOptions
        from .workflow_get_response_approval_status_options import WorkflowGetResponse_approvalStatusOptions
        from .workflow_get_response_attached_attributes import WorkflowGetResponse_attachedAttributes
        from .workflow_get_response_copy_files_options import WorkflowGetResponse_copyFilesOptions
        from .workflow_get_response_steps import WorkflowGetResponse_steps
        from .workflow_get_response_update_attributes_options import WorkflowGetResponse_updateAttributesOptions

        fields: dict[str, Callable[[Any], None]] = {
            "additionalOptions": lambda n : setattr(self, 'additional_options', n.get_object_value(WorkflowGetResponse_additionalOptions)),
            "approvalStatusOptions": lambda n : setattr(self, 'approval_status_options', n.get_collection_of_object_values(WorkflowGetResponse_approvalStatusOptions)),
            "attachedAttributes": lambda n : setattr(self, 'attached_attributes', n.get_collection_of_object_values(WorkflowGetResponse_attachedAttributes)),
            "copyFilesOptions": lambda n : setattr(self, 'copy_files_options', n.get_object_value(WorkflowGetResponse_copyFilesOptions)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "steps": lambda n : setattr(self, 'steps', n.get_collection_of_object_values(WorkflowGetResponse_steps)),
            "updateAttributesOptions": lambda n : setattr(self, 'update_attributes_options', n.get_object_value(WorkflowGetResponse_updateAttributesOptions)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("additionalOptions", self.additional_options)
        writer.write_collection_of_object_values("approvalStatusOptions", self.approval_status_options)
        writer.write_collection_of_object_values("attachedAttributes", self.attached_attributes)
        writer.write_object_value("copyFilesOptions", self.copy_files_options)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_collection_of_object_values("steps", self.steps)
        writer.write_object_value("updateAttributesOptions", self.update_attributes_options)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .clashes_assign_post_request_body_pushpin_location import ClashesAssignPostRequestBody_pushpin_location
    from .clashes_assign_post_request_body_pushpin_type import ClashesAssignPostRequestBody_pushpin_type
    from .clashes_assign_post_request_body_pushpin_viewer_state import ClashesAssignPostRequestBody_pushpin_viewerState

@dataclass
class ClashesAssignPostRequestBody_pushpin(Parsable):
    """
    An issue push pin object, which describes a visual marker to place an issue on the 3D model.
    """
    # The version of the data described in the viewer state property.
    attributes_version: Optional[int] = None
    # The external ID (for example, derived from the Revit ID) of the object in the viewer to link with this issue.
    external_id: Optional[str] = None
    # A vector describing where in 3D space the pushpin is located.
    location: Optional[ClashesAssignPostRequestBody_pushpin_location] = None
    # The ID of the object in the viewer to link with this issue.
    object_id: Optional[int] = None
    # The type of pushpin. Possible values: ``TwoDVectorPushpin``.
    type: Optional[ClashesAssignPostRequestBody_pushpin_type] = None
    # An object describing the current state of the viewer, such as the camera position.
    viewer_state: Optional[ClashesAssignPostRequestBody_pushpin_viewerState] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClashesAssignPostRequestBody_pushpin:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClashesAssignPostRequestBody_pushpin
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClashesAssignPostRequestBody_pushpin()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .clashes_assign_post_request_body_pushpin_location import ClashesAssignPostRequestBody_pushpin_location
        from .clashes_assign_post_request_body_pushpin_type import ClashesAssignPostRequestBody_pushpin_type
        from .clashes_assign_post_request_body_pushpin_viewer_state import ClashesAssignPostRequestBody_pushpin_viewerState

        from .clashes_assign_post_request_body_pushpin_location import ClashesAssignPostRequestBody_pushpin_location
        from .clashes_assign_post_request_body_pushpin_type import ClashesAssignPostRequestBody_pushpin_type
        from .clashes_assign_post_request_body_pushpin_viewer_state import ClashesAssignPostRequestBody_pushpin_viewerState

        fields: dict[str, Callable[[Any], None]] = {
            "attributesVersion": lambda n : setattr(self, 'attributes_version', n.get_int_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "location": lambda n : setattr(self, 'location', n.get_object_value(ClashesAssignPostRequestBody_pushpin_location)),
            "objectId": lambda n : setattr(self, 'object_id', n.get_int_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(ClashesAssignPostRequestBody_pushpin_type)),
            "viewerState": lambda n : setattr(self, 'viewer_state', n.get_object_value(ClashesAssignPostRequestBody_pushpin_viewerState)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("attributesVersion", self.attributes_version)
        writer.write_str_value("externalId", self.external_id)
        writer.write_object_value("location", self.location)
        writer.write_int_value("objectId", self.object_id)
        writer.write_enum_value("type", self.type)
        writer.write_object_value("viewerState", self.viewer_state)
    




from enum import Enum

class GetSortQueryParameterType(str, Enum):
    Asc = "asc",
    Desc = "desc",
    SortFieldNameDesc = "sort=fieldName [desc]",
    Name = "name",
    CreatedAt = "createdAt",
    UpdatedAt = "updatedAt",
    DisplayId = "displayId",
    SortName = "sort=name",
    SortCreatedAtDesc = "sort=createdAt desc",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_takeoff_item_get_response_property_values_source import WithTakeoffItemGetResponse_propertyValues_source
    from .with_takeoff_item_get_response_property_values_unit_of_measure import WithTakeoffItemGetResponse_propertyValues_unitOfMeasure
    from .with_takeoff_item_get_response_property_values_value import WithTakeoffItemGetResponse_propertyValues_value

@dataclass
class WithTakeoffItemGetResponse_propertyValues(Parsable):
    # Name of the property.
    name: Optional[str] = None
    # Specifies how a takeoff property value was obtained.Possible values: ``MANUAL_ENTRY``, ``BIM_PROPERTY``, ``MEASUREMENT``
    source: Optional[WithTakeoffItemGetResponse_propertyValues_source] = None
    # The unit of measurement.Possible values: ``EA``, ``IN``, ``LF``, ``YD``, ``SI``, ``SF``, ``SY``, ``CI``, ``CF``, ``CY``, ``LBS``, ``TON``, ``MM``, ``M``, ``M2``, ``M3``, ``KG``, ``T``.
    unit_of_measure: Optional[WithTakeoffItemGetResponse_propertyValues_unitOfMeasure] = None
    # The value of a takeoff instance.
    value: Optional[WithTakeoffItemGetResponse_propertyValues_value] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTakeoffItemGetResponse_propertyValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTakeoffItemGetResponse_propertyValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTakeoffItemGetResponse_propertyValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_takeoff_item_get_response_property_values_source import WithTakeoffItemGetResponse_propertyValues_source
        from .with_takeoff_item_get_response_property_values_unit_of_measure import WithTakeoffItemGetResponse_propertyValues_unitOfMeasure
        from .with_takeoff_item_get_response_property_values_value import WithTakeoffItemGetResponse_propertyValues_value

        from .with_takeoff_item_get_response_property_values_source import WithTakeoffItemGetResponse_propertyValues_source
        from .with_takeoff_item_get_response_property_values_unit_of_measure import WithTakeoffItemGetResponse_propertyValues_unitOfMeasure
        from .with_takeoff_item_get_response_property_values_value import WithTakeoffItemGetResponse_propertyValues_value

        fields: dict[str, Callable[[Any], None]] = {
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "source": lambda n : setattr(self, 'source', n.get_enum_value(WithTakeoffItemGetResponse_propertyValues_source)),
            "unitOfMeasure": lambda n : setattr(self, 'unit_of_measure', n.get_enum_value(WithTakeoffItemGetResponse_propertyValues_unitOfMeasure)),
            "value": lambda n : setattr(self, 'value', n.get_object_value(WithTakeoffItemGetResponse_propertyValues_value)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("name", self.name)
        writer.write_enum_value("source", self.source)
        writer.write_enum_value("unitOfMeasure", self.unit_of_measure)
        writer.write_object_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ChangeOrderGetResponse_splitNumber(Parsable):
    """
    Not relevant
    """
    # Not relevant
    is_free_modify: Optional[bool] = None
    # Not relevant
    number: Optional[str] = None
    # Not relevant
    postfix: Optional[str] = None
    # Not relevant
    prefix: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ChangeOrderGetResponse_splitNumber:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ChangeOrderGetResponse_splitNumber
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ChangeOrderGetResponse_splitNumber()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "isFreeModify": lambda n : setattr(self, 'is_free_modify', n.get_bool_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "postfix": lambda n : setattr(self, 'postfix', n.get_str_value()),
            "prefix": lambda n : setattr(self, 'prefix', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("isFreeModify", self.is_free_modify)
        writer.write_str_value("number", self.number)
        writer.write_str_value("postfix", self.postfix)
        writer.write_str_value("prefix", self.prefix)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.with_name_item_request_builder import WithNameItemRequestBuilder

class DataRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /data-connector/v1/accounts/{accountId}/jobs/{jobId}/data
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new DataRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/data-connector/v1/accounts/{accountId}/jobs/{jobId}/data", path_parameters)
    
    def by_name(self,name: UUID) -> WithNameItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.dataConnector.v1.accounts.item.jobs.item.data.item collection
        param name: Name of the file to retrieve
        Returns: WithNameItemRequestBuilder
        """
        if name is None:
            raise TypeError("name cannot be null.")
        from .item.with_name_item_request_builder import WithNameItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["name"] = name
        return WithNameItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .closed_get_response_groups_reason import ClosedGetResponse_groups_reason

@dataclass
class ClosedGetResponse_groups(Parsable):
    # The unique identifier of the clash test associated with the closed clash group.
    clash_test_id: Optional[UUID] = None
    # The clashes included in the closed clash group. Min items: 1 Max items: 1000.
    clashes: Optional[list[int]] = None
    # The unique identifier of the user who created the closed clash group.
    created_by: Optional[str] = None
    # The date and time that the closed clash group was created.
    created_on: Optional[datetime.datetime] = None
    # The description of the closed clash group. Max length: 1024.
    description: Optional[str] = None
    # The unique identifier of the closed clash group.
    id: Optional[UUID] = None
    # The reason for closing this clash group. Possible values: ``OTHER``, ``VALID_INTERFACE``, ``VALID_PENETRATION``, ``MINIMAL_OVERLAP``, ``ITEM_CAN_FLEX``, ``MODEL_INACCURACY``, ``FIELD_FIX``.
    reason: Optional[ClosedGetResponse_groups_reason] = None
    # The unique identifiers of screenshots associated with the closed clash group. Max items: 5.
    screen_shots: Optional[list[str]] = None
    # The title of the closed clash group. Max length: 128.
    title: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClosedGetResponse_groups:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClosedGetResponse_groups
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClosedGetResponse_groups()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .closed_get_response_groups_reason import ClosedGetResponse_groups_reason

        from .closed_get_response_groups_reason import ClosedGetResponse_groups_reason

        fields: dict[str, Callable[[Any], None]] = {
            "clashTestId": lambda n : setattr(self, 'clash_test_id', n.get_uuid_value()),
            "clashes": lambda n : setattr(self, 'clashes', n.get_collection_of_primitive_values(int)),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdOn": lambda n : setattr(self, 'created_on', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "reason": lambda n : setattr(self, 'reason', n.get_enum_value(ClosedGetResponse_groups_reason)),
            "screenShots": lambda n : setattr(self, 'screen_shots', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("clashTestId", self.clash_test_id)
        writer.write_collection_of_primitive_values("clashes", self.clashes)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("createdOn", self.created_on)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_enum_value("reason", self.reason)
        writer.write_collection_of_primitive_values("screenShots", self.screen_shots)
        writer.write_str_value("title", self.title)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .issue_types_get_response import IssueTypesGetResponse

class IssueTypesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/issues/v1/projects/{projectId}/issue-types
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new IssueTypesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/issues/v1/projects/{projectId}/issue-types{?filter%5BisActive%5D*,filter%5BupdatedAt%5D*,include*,limit*,offset*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[IssueTypesRequestBuilderGetQueryParameters]] = None) -> Optional[IssueTypesGetResponse]:
        """
        Retrieves a project's categories and types. Note the following differences in terminology between the product and the API:
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[IssueTypesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .issue_types_get_response import IssueTypesGetResponse

        return await self.request_adapter.send_async(request_info, IssueTypesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[IssueTypesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves a project's categories and types. Note the following differences in terminology between the product and the API:
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> IssueTypesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: IssueTypesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return IssueTypesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class IssueTypesRequestBuilderGetQueryParameters():
        """
        Retrieves a project's categories and types. Note the following differences in terminology between the product and the API:
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filteris_active":
                return "filter%5BisActive%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "include":
                return "include"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Filter types by status e.g. ``filter[isActive]=true`` will only return active types. Default value: ``undefined`` (meaning both active & inactive issue type categories will return).
        filteris_active: Optional[bool] = None

        # Retrieves types that were last updated at the specified date and time, in in one of the following URL-encoded formats: YYYY-MM-DDThh:mm:ss.sz or YYYY-MM-DD. Separate multiple values with commas. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03`` or ``2022-02-28T22:00:00.000Z..2022-03-28T22:00:00.000Z``- Specific day: e.g., ``2022-03-02`` or ``2022-02-28T22:00:00.000Z``- Specific start date: e.g.,  ``2022-03-02..`` or ``2022-02-28T22:00:00.000Z..``- Specific end date: e.g., ``..2022-03-02`` or ``..2022-02-28T22:00:00.000Z``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterupdated_at: Optional[str] = None

        # Use ``include=subtypes`` to include the types (subtypes) for each category (type).
        include: Optional[str] = None

        # Add ``limit=20`` to limit the results count (together with the offset to support pagination).
        limit: Optional[int] = None

        # Add ``offset=20`` to get partial results (together with the limit to support pagination).
        offset: Optional[int] = None

    
    @dataclass
    class IssueTypesRequestBuilderGetRequestConfiguration(RequestConfiguration[IssueTypesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .values_batch_update_put_request_body_custom_values_toggle_val import ValuesBatchUpdatePutRequestBody_customValues_toggleVal

@dataclass
class ValuesBatchUpdatePutRequestBody_customValues(Parsable):
    # The attribute used for updating multi-select fields. For example, ``arrayVal:`` ``[``Answer 1``, ``Answer 2``]``.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``arrayVal`` is only relevant for updating multi-select fields. To verify whether the field you want to update is a multi-select field, call GET forms and check the field's ``valueName``.
    array_val: Optional[list[str]] = None
    # The attribute used for updating single-select and dropdown fields. For example, ``choiceVal: Answer 3``.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``choiceVal`` is only relevant for updating single-select and dropdown fields. To verify whether the field you want to update is a single-select or dropdown field, call GET forms and check the field's ``valueName``.
    choice_val: Optional[str] = None
    # The attribute used for updating date fields, in the following format ``(YYYY-MM-DD)``. For example, ````dateVal``: ``1999-12-31````.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``dateVal`` is only relevant for updating date fields. To verify whether the field you want to update is a date field, call GET forms and check the field's ``valueName``.
    date_val: Optional[str] = None
    # The unique identifier of the field.
    field_id: Optional[UUID] = None
    # The name of the person who signed the form. By default, it is the name of the logged in user. It is only relevant for signature fields. To update the name you need to use the signature field ID.
    name: Optional[str] = None
    # Text for the field's notes section. This is relevant for all fields.Max length: 8000
    notes: Optional[str] = None
    # The attribute used for updating number fields. For example, ````numberVal``: ``42````.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``numberVal`` is only relevant for updating number fields. To verify whether the field you want to update is a number field, call GET forms and check the field's ``valueName``.
    number_val: Optional[float] = None
    # The attribute used for updating signature fields.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``svgVal`` is only relevant for updating signature fields. To verify whether the field you want to update is a signature field, call GET forms and check the field's ``valueName``.The signature needs to be in SVG format and it needs to be base64 encoded.
    svg_val: Optional[str] = None
    # The attribute used for updating text value fields, For example, ``textVal: This is my response!``.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``textVal`` is only relevant for updating text value fields. To verify whether the field you want to update is a text value field, call GET forms and check the field's ``valueName``.Max length: 8000
    text_val: Optional[str] = None
    # The attribute used for updating preconfigured fields. For example, ``toggleVal``: ``Yes``.Each non-tabular field is assigned a specific value type, which you need to specify when updating the field. ``toggleVal`` is only relevant for updating preconfigured fields. To verify whether the field you want to update is a preconfigured field, call GET forms and check the field's ``valueName``.Possible values: ``Yes``, ``No``, ``False``, ``True``, ``Minus``, ``Plus``, ``Fail``, ``Pass``, ``NA``
    toggle_val: Optional[ValuesBatchUpdatePutRequestBody_customValues_toggleVal] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ValuesBatchUpdatePutRequestBody_customValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ValuesBatchUpdatePutRequestBody_customValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ValuesBatchUpdatePutRequestBody_customValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .values_batch_update_put_request_body_custom_values_toggle_val import ValuesBatchUpdatePutRequestBody_customValues_toggleVal

        from .values_batch_update_put_request_body_custom_values_toggle_val import ValuesBatchUpdatePutRequestBody_customValues_toggleVal

        fields: dict[str, Callable[[Any], None]] = {
            "arrayVal": lambda n : setattr(self, 'array_val', n.get_collection_of_primitive_values(str)),
            "choiceVal": lambda n : setattr(self, 'choice_val', n.get_str_value()),
            "dateVal": lambda n : setattr(self, 'date_val', n.get_str_value()),
            "fieldId": lambda n : setattr(self, 'field_id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "numberVal": lambda n : setattr(self, 'number_val', n.get_float_value()),
            "svgVal": lambda n : setattr(self, 'svg_val', n.get_str_value()),
            "textVal": lambda n : setattr(self, 'text_val', n.get_str_value()),
            "toggleVal": lambda n : setattr(self, 'toggle_val', n.get_enum_value(ValuesBatchUpdatePutRequestBody_customValues_toggleVal)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("arrayVal", self.array_val)
        writer.write_str_value("choiceVal", self.choice_val)
        writer.write_str_value("dateVal", self.date_val)
        writer.write_uuid_value("fieldId", self.field_id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_float_value("numberVal", self.number_val)
        writer.write_str_value("svgVal", self.svg_val)
        writer.write_str_value("textVal", self.text_val)
        writer.write_enum_value("toggleVal", self.toggle_val)
    




from enum import Enum

class DocumentsGetResponse_associationType(str, Enum):
    Budget = "Budget",
    Contract = "Contract",
    ScheduleOfValue = "ScheduleOfValue",
    FormInstance = "FormInstance",
    CostItem = "CostItem",
    Payment = "Payment",
    MainContract = "MainContract",
    BudgetPayment = "BudgetPayment",
    Expense = "Expense",
    CostPayment = "CostPayment",
    ExpenseItem = "ExpenseItem",
    PaymentItem = "PaymentItem",
    OCO = "OCO",
    RCO = "RCO",
    SCO = "SCO",
    PCO = "PCO",
    RFQ = "RFQ",
    DistributionItem = "DistributionItem",
    BudgetTransfer = "BudgetTransfer",
    Fee = "Fee",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .attachments_post_response_attachments_attachment_type import AttachmentsPostResponse_attachments_attachmentType

@dataclass
class AttachmentsPostResponse_attachments(Parsable):
    # The unique identifier for the attachment, set by the client when creating the attachment reference. This can be any unique GUID, but it is recommended to use the OSS storage GUID.
    attachment_id: Optional[UUID] = None
    # The type of attachment. For issue attachments, this value is always ``issue-attachment``. Will always be: ``issue-attachment``
    attachment_type: Optional[AttachmentsPostResponse_attachments_attachmentType] = None
    # Not relevant
    bubble_urn: Optional[str] = None
    # The ID of the user who created the issue attachment. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    created_by: Optional[str] = None
    # The date and time when the issue attachment was created, in ISO8601 format.
    created_on: Optional[datetime.datetime] = None
    # The ID of the user who deleted the issue attachment, if applicable. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    deleted_by: Optional[str] = None
    # The date and time when the issue attachment was deleted, if applicable.
    deleted_on: Optional[datetime.datetime] = None
    # The human-readable display name for the attachment, including the file extension (for example, ``.pdf``, ``.jpg``, ``.dwg``). This name appears in the ACC web UI and is used when downloading the file from the issue.
    display_name: Optional[str] = None
    # The ID of the issue that owns the attachment.
    domain_entity_id: Optional[UUID] = None
    # The unique filename of the attachment, as stored in Autodesk Docs (OSS).This is the name assigned to the uploaded file in the system, typically formatted as {attachmentId}.{fileExtension}.
    file_name: Optional[str] = None
    # The size of the file in bytes.
    file_size: Optional[int] = None
    # The file extension (without the dot), for example ``pdf`` or ``jpg``.
    file_type: Optional[str] = None
    # ``true``: The attachment has been deleted.``false``: (default) The attachment has not been deleted.
    is_deleted: Optional[bool] = None
    # The document lineage URN for the attachment’s source file.
    lineage_urn: Optional[str] = None
    # Not relevant
    modified_by: Optional[str] = None
    # Not relevant
    modified_on: Optional[datetime.datetime] = None
    # The Object Storage Service (OSS) URN identifying where the attachment file is stored in Autodesk’s cloud infrastructure. Use this value when downloading the file (see the Download Issue Attachment </en/docs/acc/v1/tutorials/issues/download-issue-attachments/>_ tutorial).
    storage_urn: Optional[str] = None
    # The URN for the latest (tip) version of the file.
    tip_version_urn: Optional[str] = None
    # The document version number.
    version: Optional[int] = None
    # The URN for the specific file version that was attached to the issue. This may differ from the latest version URN (``tipVersionUrn``) if a newer version of the file exists in Autodesk Docs.
    version_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttachmentsPostResponse_attachments:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttachmentsPostResponse_attachments
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttachmentsPostResponse_attachments()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attachments_post_response_attachments_attachment_type import AttachmentsPostResponse_attachments_attachmentType

        from .attachments_post_response_attachments_attachment_type import AttachmentsPostResponse_attachments_attachmentType

        fields: dict[str, Callable[[Any], None]] = {
            "attachmentId": lambda n : setattr(self, 'attachment_id', n.get_uuid_value()),
            "attachmentType": lambda n : setattr(self, 'attachment_type', n.get_enum_value(AttachmentsPostResponse_attachments_attachmentType)),
            "bubbleUrn": lambda n : setattr(self, 'bubble_urn', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdOn": lambda n : setattr(self, 'created_on', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "deletedOn": lambda n : setattr(self, 'deleted_on', n.get_datetime_value()),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "domainEntityId": lambda n : setattr(self, 'domain_entity_id', n.get_uuid_value()),
            "fileName": lambda n : setattr(self, 'file_name', n.get_str_value()),
            "fileSize": lambda n : setattr(self, 'file_size', n.get_int_value()),
            "fileType": lambda n : setattr(self, 'file_type', n.get_str_value()),
            "isDeleted": lambda n : setattr(self, 'is_deleted', n.get_bool_value()),
            "lineageUrn": lambda n : setattr(self, 'lineage_urn', n.get_str_value()),
            "modifiedBy": lambda n : setattr(self, 'modified_by', n.get_str_value()),
            "modifiedOn": lambda n : setattr(self, 'modified_on', n.get_datetime_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
            "tipVersionUrn": lambda n : setattr(self, 'tip_version_urn', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
            "versionUrn": lambda n : setattr(self, 'version_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("attachmentId", self.attachment_id)
        writer.write_enum_value("attachmentType", self.attachment_type)
        writer.write_str_value("bubbleUrn", self.bubble_urn)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("createdOn", self.created_on)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_datetime_value("deletedOn", self.deleted_on)
        writer.write_str_value("displayName", self.display_name)
        writer.write_uuid_value("domainEntityId", self.domain_entity_id)
        writer.write_str_value("fileName", self.file_name)
        writer.write_int_value("fileSize", self.file_size)
        writer.write_str_value("fileType", self.file_type)
        writer.write_bool_value("isDeleted", self.is_deleted)
        writer.write_str_value("lineageUrn", self.lineage_urn)
        writer.write_str_value("modifiedBy", self.modified_by)
        writer.write_datetime_value("modifiedOn", self.modified_on)
        writer.write_str_value("storageUrn", self.storage_urn)
        writer.write_str_value("tipVersionUrn", self.tip_version_urn)
        writer.write_int_value("version", self.version)
        writer.write_str_value("versionUrn", self.version_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .fields_get_response_type import FieldsGetResponse_type

@dataclass
class FieldsGetResponse(Parsable):
    # The property database attribute category, can be null.
    category: Optional[str] = None
    # The unique identifier for the attribute in file version.
    key: Optional[str] = None
    # The property database attribute name.
    name: Optional[str] = None
    # field data type.Possible values: ``Unknown``, ``Boolean``, ``Integer``, ``Double``, ``Blob``, ``DbKey``, ``String``, ``LocalizableString``, ``DateTime``, ``GeoLocation``, ``Position``
    type: Optional[FieldsGetResponse_type] = None
    # The property database attribute data type context or unit of measurement, e.g., ``m``, ``ft``, ``m^2``, ``kip/inch^2``.
    uom: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> FieldsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: FieldsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return FieldsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .fields_get_response_type import FieldsGetResponse_type

        from .fields_get_response_type import FieldsGetResponse_type

        fields: dict[str, Callable[[Any], None]] = {
            "category": lambda n : setattr(self, 'category', n.get_str_value()),
            "key": lambda n : setattr(self, 'key', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(FieldsGetResponse_type)),
            "uom": lambda n : setattr(self, 'uom', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("category", self.category)
        writer.write_str_value("key", self.key)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("type", self.type)
        writer.write_str_value("uom", self.uom)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .thumbnails_batch_get_post_request_body_type import ThumbnailsBatchGetPostRequestBody_type

@dataclass
class ThumbnailsBatchGetPostRequestBody(Parsable):
    # The IDs of the review sheets you want to get the thumbnails from.To find the review sheet IDs, call `GET review-sheets </en/docs/acc/v1/reference/http/sheets-review-sheets-GET/>`_.- The max number of items is 100.
    review_sheet_ids: Optional[list[str]] = None
    # The size type of the thumbnails. Possible values:- ``big``: the max size will be 512 pixels.- ``small``: the max size will be 256 pixels.- ``tiny``: the max size will be 64 pixels.
    type: Optional[ThumbnailsBatchGetPostRequestBody_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ThumbnailsBatchGetPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ThumbnailsBatchGetPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ThumbnailsBatchGetPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .thumbnails_batch_get_post_request_body_type import ThumbnailsBatchGetPostRequestBody_type

        from .thumbnails_batch_get_post_request_body_type import ThumbnailsBatchGetPostRequestBody_type

        fields: dict[str, Callable[[Any], None]] = {
            "reviewSheetIds": lambda n : setattr(self, 'review_sheet_ids', n.get_collection_of_primitive_values(str)),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(ThumbnailsBatchGetPostRequestBody_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("reviewSheetIds", self.review_sheet_ids)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_takeoff_type_get_response_primary_quantity_definition_unit_of_measure import WithTakeoffTypeGetResponse_primaryQuantityDefinition_unitOfMeasure

@dataclass
class WithTakeoffTypeGetResponse_primaryQuantityDefinition(Parsable):
    """
    The classification quantity details.
    """
    # The classification code selected from the first classification system.
    classification_code_one: Optional[str] = None
    # The classification code selected from the second classification system.
    classification_code_two: Optional[str] = None
    # The formula to calculate the quantity.
    expression: Optional[str] = None
    # A custom output name from the user.
    output_name: Optional[str] = None
    # The unit of measurement.Possible values: ``EA``, ``IN``, ``LF``, ``YD``, ``SI``, ``SF``, ``SY``, ``CI``, ``CF``, ``CY``, ``LBS``, ``TON``, ``MM``, ``M``, ``M2``, ``M3``, ``KG``, ``T``.
    unit_of_measure: Optional[WithTakeoffTypeGetResponse_primaryQuantityDefinition_unitOfMeasure] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTakeoffTypeGetResponse_primaryQuantityDefinition:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTakeoffTypeGetResponse_primaryQuantityDefinition
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTakeoffTypeGetResponse_primaryQuantityDefinition()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_takeoff_type_get_response_primary_quantity_definition_unit_of_measure import WithTakeoffTypeGetResponse_primaryQuantityDefinition_unitOfMeasure

        from .with_takeoff_type_get_response_primary_quantity_definition_unit_of_measure import WithTakeoffTypeGetResponse_primaryQuantityDefinition_unitOfMeasure

        fields: dict[str, Callable[[Any], None]] = {
            "classificationCodeOne": lambda n : setattr(self, 'classification_code_one', n.get_str_value()),
            "classificationCodeTwo": lambda n : setattr(self, 'classification_code_two', n.get_str_value()),
            "expression": lambda n : setattr(self, 'expression', n.get_str_value()),
            "outputName": lambda n : setattr(self, 'output_name', n.get_str_value()),
            "unitOfMeasure": lambda n : setattr(self, 'unit_of_measure', n.get_enum_value(WithTakeoffTypeGetResponse_primaryQuantityDefinition_unitOfMeasure)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("classificationCodeOne", self.classification_code_one)
        writer.write_str_value("classificationCodeTwo", self.classification_code_two)
        writer.write_str_value("expression", self.expression)
        writer.write_str_value("outputName", self.output_name)
        writer.write_enum_value("unitOfMeasure", self.unit_of_measure)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .action_histories_get_response_results_operator import ActionHistoriesGetResponse_results_operator
    from .action_histories_get_response_results_options import ActionHistoriesGetResponse_results_options

@dataclass
class ActionHistoriesGetResponse_results(Parsable):
    # The specific action that occurred (e.g., ``review:proceed``, ``submitted``, ``rejected``).
    action: Optional[str] = None
    # The ID of the object associated with the action history.
    association_id: Optional[str] = None
    # The type of object associated with the action history (e.g., ``Contract``, ``SCO``)
    association_type: Optional[str] = None
    # The timestamp when the action occurred.
    created_at: Optional[datetime.datetime] = None
    # Information about the user who performed the action.
    operator: Optional[ActionHistoriesGetResponse_results_operator] = None
    # Additional context for the action. The fields present in this object depend on the action history type.For Approval workflows, the object includes:``stepName`` – the name of the workflow step.``taskDefinitionKey`` – the key that identifies the workflow task.``workflowInstanceId`` – the ID of the workflow instance.These fields provide task-level process information.For Normal workflows, the object may include fields such as to and from that indicate a change in status.
    options: Optional[ActionHistoriesGetResponse_results_options] = None
    # The category of the action history. Possible values: ``Approval``, ``Normal``.
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ActionHistoriesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ActionHistoriesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ActionHistoriesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .action_histories_get_response_results_operator import ActionHistoriesGetResponse_results_operator
        from .action_histories_get_response_results_options import ActionHistoriesGetResponse_results_options

        from .action_histories_get_response_results_operator import ActionHistoriesGetResponse_results_operator
        from .action_histories_get_response_results_options import ActionHistoriesGetResponse_results_options

        fields: dict[str, Callable[[Any], None]] = {
            "action": lambda n : setattr(self, 'action', n.get_str_value()),
            "associationId": lambda n : setattr(self, 'association_id', n.get_str_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "operator": lambda n : setattr(self, 'operator', n.get_object_value(ActionHistoriesGetResponse_results_operator)),
            "options": lambda n : setattr(self, 'options', n.get_object_value(ActionHistoriesGetResponse_results_options)),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("action", self.action)
        writer.write_str_value("associationId", self.association_id)
        writer.write_str_value("associationType", self.association_type)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_object_value("operator", self.operator)
        writer.write_object_value("options", self.options)
        writer.write_str_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithSegmentPatchRequestBody(Parsable):
    # The delimiter that follows the segment. Possible values are: ``none``, ``space``, ``point``, ``hyphen``, ``underscore``, ``tab``.
    delimiter: Optional[str] = None
    # ``true`` Lock the segment.``false`` Do not lock the segment.
    is_locked: Optional[bool] = None
    # The number of characters allowed in the segment.
    length: Optional[float] = None
    # The name of the segment in budget code template.Max length: 1024
    name: Optional[str] = None
    # A code sample for the segment used to demonstrate how the segment looks when displayed.
    sample_code: Optional[str] = None
    # The segment type. Possible values: ``code``, ``column``, or ``info``. Code segments are displayed as part of the budget code. Column segments are displayed in a separate column. Info segments are not displayed.
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithSegmentPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithSegmentPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithSegmentPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "delimiter": lambda n : setattr(self, 'delimiter', n.get_str_value()),
            "isLocked": lambda n : setattr(self, 'is_locked', n.get_bool_value()),
            "length": lambda n : setattr(self, 'length', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "sampleCode": lambda n : setattr(self, 'sample_code', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("delimiter", self.delimiter)
        writer.write_bool_value("isLocked", self.is_locked)
        writer.write_float_value("length", self.length)
        writer.write_str_value("name", self.name)
        writer.write_str_value("sampleCode", self.sample_code)
        writer.write_str_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_rfi_get_response_permitted_actions_update_rfi_permitted_statuses_wf_e_u import WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfEU
    from .with_rfi_get_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s import WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfUS

@dataclass
class WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses(Parsable):
    """
    A list of statuses the user is permitted to transition an RFI to.
    """
    # A list of statuses the user is permitted to transition an RFI to in workflows of type ``EMEA``.
    wf_e_u: Optional[list[WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfEU]] = None
    # A list of statuses the user is permitted to transition an RFI to in workflows of type ``US``.
    wf_u_s: Optional[list[WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfUS]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_rfi_get_response_permitted_actions_update_rfi_permitted_statuses_wf_e_u import WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfEU
        from .with_rfi_get_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s import WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfUS

        from .with_rfi_get_response_permitted_actions_update_rfi_permitted_statuses_wf_e_u import WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfEU
        from .with_rfi_get_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s import WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfUS

        fields: dict[str, Callable[[Any], None]] = {
            "wfEU": lambda n : setattr(self, 'wf_e_u', n.get_collection_of_object_values(WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfEU)),
            "wfUS": lambda n : setattr(self, 'wf_u_s', n.get_collection_of_object_values(WithRfiGetResponse_permittedActions_updateRfi_permittedStatuses_wfUS)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("wfEU", self.wf_e_u)
        writer.write_collection_of_object_values("wfUS", self.wf_u_s)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_view_delete_response_job_seed import WithViewDeleteResponse_job_seed

@dataclass
class WithViewDeleteResponse_job(Parsable):
    """
    A job.
    """
    # The operation associated with the job.
    operation: Optional[str] = None
    # The JSON payload which seeded the job.
    seed: Optional[WithViewDeleteResponse_job_seed] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithViewDeleteResponse_job:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithViewDeleteResponse_job
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithViewDeleteResponse_job()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_view_delete_response_job_seed import WithViewDeleteResponse_job_seed

        from .with_view_delete_response_job_seed import WithViewDeleteResponse_job_seed

        fields: dict[str, Callable[[Any], None]] = {
            "operation": lambda n : setattr(self, 'operation', n.get_str_value()),
            "seed": lambda n : setattr(self, 'seed', n.get_object_value(WithViewDeleteResponse_job_seed)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("operation", self.operation)
        writer.write_object_value("seed", self.seed)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_folder_item_request_builder import WithFolder_ItemRequestBuilder

class FoldersRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/docs/v1/projects/{project-id}/folders
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new FoldersRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/docs/v1/projects/{project%2Did}/folders", path_parameters)
    
    def by_folder_id(self,folder_id: str) -> WithFolder_ItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.bim360.docs.v1.projects.item.folders.item collection
        param folder_id: The ID (URN) of the folder.For details about how to find the URN, follow the initial steps (1-3) in the `Download Files </en/docs/bim360/v1/tutorials/document-management/download-document-s3/>`_ tutorial.
        Returns: WithFolder_ItemRequestBuilder
        """
        if folder_id is None:
            raise TypeError("folder_id cannot be null.")
        from .item.with_folder_item_request_builder import WithFolder_ItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["folder_id"] = folder_id
        return WithFolder_ItemRequestBuilder(self.request_adapter, url_tpl_params)
    

